


#include <DS1307.h>
#include <Wire.h>



#define ISRPIN 3 // use pin 2 or 3 for external interrupt



#define DebugPrint(p) 
//Serial.println(p)

// you have to forward declare the "callback" functions
extern void SecondsLeap(byte secs);
extern void MinuteLeap(byte h,byte m,byte d,byte mon,unsigned short year,byte wof);

#define DCF77_LED_REPLICAPIN -1    // -1 to disable LED replica, note that an active LED can influence the DCF77 module in weired ways
#define DCF77_PARTIALOK false      // accept partial results
#define DCF77_MAXMOVINGAVERAGE 3   // in seconds, set to 0 to disable
#define DCF77_MINDRIFTCORRECTION 3 // maxdrift allowed before correcting the drift in msecs
#define DCF77_Pin 2                // pin to which DCF77 board connected
#define DCF77_INTNR 0              // interrupt number (usually 0 if pin=2 and 1 if pin=3)
#define DCF77_MINUTECALLBACK(h,m,d,mon,year,wof) MinuteLeap(h,m,d,mon,year,wof)
#define DCF77_SECONDCALLBACK(sec)  
#define DCF77_MSECSVARIATION 35

#include <DCF77ISR.h>


volatile bool newClock=false;
int clockState=0;
int rtc[7];


int lastState=-1;
unsigned long nextStats=0;

void RTCsetDateTime(int h, int m, int s, int day, int month, int year)
{
	RTC.stop();
	RTC.set(DS1307_HR, h);
	RTC.set(DS1307_MIN, m);
	RTC.set(DS1307_SEC, s);
	RTC.set(DS1307_DATE, day);
	RTC.set(DS1307_MTH, month);
	RTC.set(DS1307_YR, year);
	RTC.start();
	
}


bool setNow=false;
byte _h,_m,_d,_mon, _wof;
unsigned short _year;

void MinuteLeap(byte h,byte m,byte d,byte mon,unsigned short year,byte wof)
{
	
	char outb[40];
	// update RTC clock if different
	if (d==0)
		return;
	if (mon==0)
		return;
	if (year==0)
		return;
	{
		_h=h;
		_m=m;
		_d=d;
		_mon=mon;
		_year=year;
		_wof=wof;
		setNow=true;
	}
	sprintf(outb,"DCF77Time=%2d:%2d %2d/%2d/%4d (%d)", h, m, d, mon, year, wof);
	Serial.println(outb);
}



// with 1hz squarewave this routine will be called 2 times per second if using CHANGE (attachInterrupt)
// with RISING it will be called 1 time per second
void DS1307Callback()
{
	clockState=digitalRead(ISRPIN);
	newClock=true;
}

void RTCshowDateTime()
{
	char outb[64];
	RTC.get(rtc, true);
	//if (rtc[0]%10==0)
	{
		sprintf(outb,"%02d:%02d:%02d %2d/%2d/%4d day=%d ",rtc[2],rtc[1],rtc[0],rtc[4],rtc[5],rtc[6],rtc[3]);
		Serial.println(outb);
	}
}


void setup(void)
{
    DCF77ISR::setup(); // pin, int
    Serial.begin(9600);
	pinMode(ISRPIN,INPUT);
	// pull-up ISRPIN (see DS1307 datasheet)
	digitalWrite(ISRPIN,HIGH); 
	// set external interrupt for 2Hz interruption
	attachInterrupt(ISRPIN-2, DS1307Callback, CHANGE);
	RTCshowDateTime();
	RTC.set_ctrlreg(DS1307_1HZ);
	Serial.println("running (i hope)");
}

void loop(void) {
	// RTC show
	if (newClock) // show new time only when new interrupt signaled
	{
		newClock=false;
		//Serial.print(clockState); // clockstate is useful if you want to do a blinking ':' for the time
		//Serial.print(" ");
		//RTCshowDateTime();
	}
	
	// stuff for DCF77: show stats
	/* 
	if (millis()>nextStats)
    {
        nextStats=millis()+15000;
 		char outb[64];
		sprintf(outb,"good=%3d bad=%4d assigned=%2d double=%d doubt=%d drift=%4d ", DCF77ISR::goodCnt, DCF77ISR::badCnt, DCF77ISR::assignCnt, DCF77ISR::assignDoubleCnt, DCF77ISR::doubtAssigned,DCF77ISR::driftPerMinute);
		Serial.print(outb);
		//Serial.println(DCF77ISR::bitAssigned); 
		RTCshowDateTime();      
    }
    */
    if (setNow)
    {
    	setNow=false;
		RTCshowDateTime();
    	if ((_h!=rtc[2])|| (_m!=rtc[1]) || (0!=rtc[0]) || (_d!=rtc[4])|| (_mon!=rtc[5]) || (_year!=rtc[6]))
		{
			RTCsetDateTime(_h, _m, 1, _d, _mon, _year);
			Serial.println("**** TIME UPDATED ON RTC ****");
		}
		else
		{
			Serial.println("**** TIME IS OK ON RTC ****");
		}
	}
	if (DCF77ISR::assigned!=DCF77ISR::MA_NONE)
	{
		RTCshowDateTime();
		nextStats=millis()+15000;
		if (DCF77ISR::assigned==DCF77ISR::MA_BAD)
			Serial.println("not assigned, some bad data!");
		char outb[64];
		sprintf(outb,"good=%3d bad=%4d assigned=%2d double=%d doubt=%d drift=%4d msecs/min", DCF77ISR::goodCnt, DCF77ISR::badCnt, DCF77ISR::assignCnt, DCF77ISR::assignDoubleCnt, DCF77ISR::doubtAssigned,DCF77ISR::driftPerMinute);
		Serial.println (outb);
		sprintf(outb,"%2d:%2d %2d/%2d/%4d (%d)", DCF77ISR::hour, DCF77ISR::min, DCF77ISR::day, DCF77ISR::month, DCF77ISR::year, DCF77ISR::weekday);
		Serial.println(outb);
		Serial.println("bits:");
		Serial.println(DCF77ISR::bitAssigned);
		DCF77ISR::ResetMinuteStuff();
	}
}

